require 'sinatra/base'
require 'lib/sinatra_helpers'
require 'lib/console_auth'
require 'yaml'
require 'active_record'
require 'mail'

class ConsoleAuthApp < Sinatra::Base

  helpers SinatraHelpers

  configure :development, :test do
    enable :raise_errors
  end

  get '/admin' do
    send_file 'views/admin.html'
  end

  get %r[^(/views/.*)] do
    send_file "batman" + params[:captures].first
  end

  get '/activate/:key' do
    cache_control :no_cache
    @min_length = ConsoleAuth.get_config['password']['min_length']
    erb :activate
  end

  get '/account' do
    cache_control :no_cache
    @min_length = ConsoleAuth.get_config['password']['min_length']
    @user = ConsoleAuth.get_user(env)
    return [ 400, "No user was found."] unless @user
    erb :account
  end

  get '/users' do
    cache_control :no_cache
    { 'users' => user_info(User.all, ConsoleAuth.get_user(env)) }.to_json
  end

  get '/users/:id' do
    cache_control :no_cache
    user = User.find_by_id(params[:id])
    return [ 404, "No user with id #{params[:id]} was found" ] unless user
    user_info([user]).first.to_json
  end

  post '/users' do
    cache_control :no_cache
    content_type :json
    begin
      admin = ConsoleAuth.get_user(env)
      return [ 400, "You must be an admin to do that!" ] unless admin.roles.first.name == "admin"
      role = Role.find_by_name(params[:user][:role])
      return [ 404, "No role with name #{params[:user][:role]} was found" ] unless role
      unless params[:user][:role] && params[:user][:email]
        return [ 400, "Required information for creating a user (email, role) was not passed." ]
      end
      user = User.new({ :email => params[:user][:email],
                        :roles => [role],
                        :status => "pending" })
      user.password = ''
      user.save!
      ConsoleAuth.logger.info("User #{user.id} was created by admin #{admin.id} (#{admin.email}).")
      activation = create_activation(user, admin.email)
      send_confirmation_email(user, admin.email, activation)
    rescue Exception => e
      ConsoleAuth.logger.error("An error occured while creating user #{params[:user][:email]}: #{e.message}")
      ConsoleAuth.logger.error("Stack trace: #{e.backtrace}")
      error_message = e.message =~ /has already been taken/ ? "User already exists" : e.message
      return [ 400, error_message ]
    end
    info = user_info([user]).first
    info['activation_link'] = activation_link(activation.key)
    info.to_json
  end

  delete '/users/:id' do
    cache_control :no_cache
    begin
      admin = ConsoleAuth.get_user(env)
      return [ 400, "You must be an admin to do that!" ] unless admin.roles.first.name == "admin"
      user = User.find_by_id(params[:id])
      return [ 404, "No user with id #{params[:id]} was found" ] unless user
      return [ 400, "Admins cannot delete themselves" ] if admin.id == user.id
      if user.status == "pending" || user.status == "expired"
        activation = Activation.find_by_user_id(user.id)
        activation.delete
      end
      user.delete
      ConsoleAuth.logger.info("User #{params[:id]} was deleted by admin #{admin.id} (#{admin.email}).")
    rescue Exception => e
      ConsoleAuth.logger.error("An error occured while deleting user #{params[:id]} (#{params[:email]}): #{e.message}")
      return [ 400, e.message ]
    end
    "User was deleted.".to_json
  end

  put '/users/:id' do
    cache_control :no_cache
    begin
      activation = nil
      admin = ConsoleAuth.get_user(env)
      return [ 400, "You must be an admin to do that!" ] unless admin.roles.first.name == "admin"
      user = User.find_by_id(params[:id])
      return [ 404, "No user with id #{params[:id]} was found" ] unless user
      if admin.id == user.id && (params[:user][:status] != user.status)
        return [ 400, "Admins cannot disable themselves" ]
      end
      user.attributes.each do |attr, val|
        param_val = params[:user][attr.to_sym]
        next unless param_val && val.to_s != param_val.to_s
        user.update_attribute(attr, param_val)
        if attr == "email"
          activation = create_activation(user, admin.email)
          send_confirmation_email(user, admin.email, activation)
          user.update_attribute("status","pending")
        end
        ConsoleAuth.logger.info("User #{user.id} (#{user.email}) was updated by admin #{admin.id} (#{admin.email}): #{attr} changed from #{val} to #{param_val}")
      end
      old_role = user.roles.first.name
      if params[:user][:role] && params[:user][:role] != old_role
        return [ 400, "Admins cannot change their own role" ] if admin.id == user.id
        user.roles = [Role.find_by_name(params[:user][:role])]
        user.save!
        ConsoleAuth.logger.info("User #{user.id} (#{user.email}) was updated by admin #{admin.id} (#{admin.email}): role changed from #{old_role} to #{user.roles.first.name}")
      end
    rescue Exception => e
      ConsoleAuth.logger.error("An error occured while attempting to update user #{user.id} (#{user.email}): #{e.inspect}")
      ConsoleAuth.logger.error("Stacktrace: #{e.backtrace}")
      return [ 400, e.message ]
    end
    info = user_info([user]).first
    info['activation_link'] = activation_link(activation.key) if activation
    info.to_json
  end

  post '/unset_password' do
    cache_control :no_cache
    content_type :json
    begin
      current_user = ConsoleAuth.get_user(env)
      return [ 400, "You must be an admin to do that!" ] unless current_user.roles.first.name == "admin"
      return [ 400, "Missing parameter: user_id"] unless params[:user_id]
      target_user = User.find_by_id(params[:user_id])
      target_user.password = ''
      target_user.status = "pending"
      target_user.save!
      ConsoleAuth.logger.info("User #{target_user.id} (#{target_user.email}) had their password reset by admin #{current_user.id} (#{current_user.email})")
      activation = create_activation(target_user, current_user.email)
      send_confirmation_email(target_user, current_user.email, activation)
      result = user_info([target_user]).first
      result['activation_link'] = activation_link(activation.key)
      return result.to_json
    rescue Exception => e
      ConsoleAuth.logger.error("An error occured when #{current_user.id} (#{current_user.email}) attempted to unset the password for user #{target_user.id} (#{target_user.email}): #{e.inspect}")
      ConsoleAuth.logger.error("Stacktrace: #{e.backtrace}")
      return [ 400, e.message ]
    end
  end

  post '/reset_password' do
    cache_control :no_cache
    content_type :json
    begin
      current_user = ConsoleAuth.get_user(env)
      new_password = params[:password]
      return [ 400, "Missing parameter: password"] unless new_password
      min_length = ConsoleAuth.get_config['password']['min_length']
      return [ 400, "Password must be #{min_length} characters or longer." ] if new_password.length < min_length
      current_user.password = new_password
      current_user.save!
      ConsoleAuth.logger.info("User #{current_user.id} (#{current_user.email}) reset their password.")
      return "Password successfully reset.".to_json
    rescue Exception => e
      ConsoleAuth.logger.error("An error occured when #{current_user.id} (#{current_user.email}) attempted to reset their password: #{e.inspect}")
      ConsoleAuth.logger.error("Stacktrace: #{e.backtrace}")
      return [ 400, e.message ]
    end
  end

  get '/activation/:key' do
    cache_control :no_cache
    content_type :json
    activation = Activation.find_by_key(params[:key])
    return [ 404, "Invalid activation link"] unless activation
    user = User.find_by_id(activation.user_id)
    return [ 400, "No user with id #{activation.user_id} was found corresponding to activation record #{activation.id}"] unless user
    if (user.status == "pending") && fresh?(activation)
      return user_info([user]).first.to_json
    else
      user.status = "expired"
      user.save!
      ConsoleAuth.logger.info("User #{user.id} (#{user.email}) attempted to activate an expired account; delete the account and create a new one.")
      return [ 400, "Activation has expired; notify your admin to create a new account for you." ]
    end
  end

  post '/activation/:key' do
    cache_control :no_cache
    begin
      activation = Activation.find_by_key(params[:key])
      return [ 404, "Invalid activation link"] unless activation
      user = User.find_by_id(activation.user_id)
      return [ 404, "No user with id #{activation.user_id} was found corresponding to activation record #{activation.id}"] unless user
      password = params[:password]
      return [ 400, "Missing parameter: password"] unless password
      min_length = ConsoleAuth.get_config['password']['min_length']
      return [ 400, "Password must be #{min_length} characters or longer." ] if password.length < min_length
      if (user.status == "pending") && fresh?(activation)
        user.status = "enabled"
        user.password = password
        user.save!
        activation.delete
        ConsoleAuth.logger.info("User #{user.id} (#{user.email}) was activated")
      else
        user.status = "expired"
        user.save!
        ConsoleAuth.logger.info("User #{user.id} (#{user.email}) attempted to activate an expired account; delete the account and create a new one.")
        return [ 400, "Activation has expired; notify your admin to create a new account for you." ]
      end
    rescue Exception => e
      ConsoleAuth.logger.error("Activating user with activation key #{params[:key]} failed: #{e.inspect}")
      return [ 400, "Activating the user failed: #{e.message}" ]
    end
    "User has been activated.".to_json
  end
end
