require 'spec_helper'

describe BaselineUtil do
  describe ".create_missing_baselines_for_node" do
    it "should raise if the node has no inspect reports" do
      node = Factory :node
      expect { BaselineUtil.create_missing_baselines_for_node node }.
        should raise_error /without an inspect report/
    end

    context "with an inspect report" do
      before :each do
        @report = Factory :inspect_report
        @report.time = Time.now
        @report.save

        @node = @report.node
      end

      it "should bootstrap a baseline if the report is from today" do
        NodeBaseline.all.count.should == 0
        BaselineUtil.create_missing_baselines_for_node @node
        NodeBaseline.all.count.should == 1
        NodeBaseline.first.node_id.should == @node.id
      end

      it "should date the bootstrap baseline *today*" do
        pending "Waiting for decision on #8998"

        BaselineUtil.create_missing_baselines_for_node @node
        NodeBaseline.first.date.should == BaselineUtil.day_ending(Date.today).to_date
      end

      it "should bootstrap a baseline and fill in NodeBaselines if the report is earlier than today" do
        @report.time = Date.today + 4.hours - 5.days
        @report.save

        NodeBaseline.all.count.should == 0
        BaselineUtil.create_missing_baselines_for_node @node

        NodeBaseline.all.count.should == 5
        NodeBaseline.all.map(&:node_id).should == [@node.id] * 5
      end

      it "should fill in NodeBaselines if a NodeBaseline exists earlier than today" do
        @report.time = Date.today + 4.hours - 5.days
        @report.save

        nb = NodeBaseline.
          create_from_report!(@report, BaselineUtil.day_ending(@report.time))
        nb.date.should == BaselineUtil.day_ending(@report.time).to_date

        NodeBaseline.all.count.should == 1
        BaselineUtil.create_missing_baselines_for_node @node

        NodeBaseline.all.count.should == 5
        NodeBaseline.all.map(&:node_id).should == [@node.id] * 5
      end

      it "should do nothing if there is a NodeBaseline for today" do
        nb = NodeBaseline.
          create_from_report!(@report, BaselineUtil.day_ending(@report.time))

        NodeBaseline.all.count.should == 1
        BaselineUtil.create_missing_baselines_for_node @node

        NodeBaseline.all.count.should == 1
        NodeBaseline.all.map(&:node_id).should == [@node.id]
      end
    end

    it "(#9214) should roll changes into the immediate next days baselines" do
      # Should verify if that is actually required.
      SETTINGS.baseline_day_end = '0500'

      node     = Node.generate!
      report   = Factory(:inspect_report, :host => 'bomf',
                         :time => Time.local(2012, 1, 1, 10, 0, 0))
      resource = Factory(:resource_status, :report => report, :title => '/root')
      Factory(:resource_event, :resource_status => resource,
              :property => 'mode', :previous_value => '0700')

      # OK, now we generate the NodeBaseline record for 0530 the next day.
      nb = NodeBaseline.
        create_from_report!(report, BaselineUtil.day_ending(report.time))
      Summarizer.build_baseline_summary(report.id, nb)

      # ...and now the report, for the next evening at 10, showing a change to
      # the /root directory.
      report   = Factory(:inspect_report, :host => 'bomf',
                         :time => Time.local(2012, 1, 2, 10, 0, 0))
      resource = Factory(:resource_status, :report => report, :title => '/root')
      Factory(:resource_event, :resource_status => resource,
              :property => 'mode', :previous_value => '0750')

      # Create the next node_baseline record.  At 2012-01-02 05:30
      nb = nb.create_for_next_day
      nb.date.day.should == 2
      Summarizer.build_baseline_summary(report.id, nb)

      # We should now have a difference recorded.
      BaselineDifference.count.should == 1

      # Approve the change at a point in time after that, around 8AM.
      time = Time.local(2012, 1, 2, 12, 0, 0)
      BaselineDifference.first.update_attributes! :accepted => true, :decided_at => time

      # Create another report, showing nothing has changed from yesterday, but
      # it has changed from the initial baseline.
      report   = Factory(:inspect_report, :host => 'bomf',
                         :time => Time.local(2012, 1, 3, 10, 0, 0))
      resource = Factory(:resource_status, :report => report, :title => '/root')
      Factory(:resource_event, :resource_status => resource,
              :property => 'mode', :previous_value => '0750')

      # Create the next baseline.
      nb = nb.create_for_next_day
      nb.date.day.should == 3
      Summarizer.build_baseline_summary(report.id, nb)

      # ...and we should *not* have another difference reported.
      BaselineDifference.count.should == 1
    end
  end

  describe ".other_day_summaries" do
    before :each do
      @date = Date.today
      node = Node.generate!
      @today     = NodeBaseline.create!(:date => @date, :node_id => node.id, :reference_node_id => node.id)
      @yesterday = NodeBaseline.create!(:date => @date - 1.day, :node_id => node.id, :reference_node_id => node.id)
      @old       = NodeBaseline.create!(:date => @date - 2.days, :node_id => node.id, :reference_node_id => node.id)
    end

    it "should return the count of unreviewed changes on all other days" do
      2.times { @today.differences.create! }
      3.times { @yesterday.differences.create! }
      4.times { @old.differences.create! }
      5.times { @old.differences.create!(:accepted => true, :decided_at => Time.now) }

      BaselineUtil.other_day_summaries(@date).should == [
        ["#{@yesterday.date.to_s(:db)} (3 unreviewed)", @yesterday.date.to_s(:db)],
        ["#{@old.date.to_s(:db)} (4 unreviewed)", @old.date.to_s(:db)],
      ]
    end

    it "should limit the count to particular node baselines if specified" do
      other_node = Node.generate!
      yesterday_other = NodeBaseline.create!(:date => @yesterday.date, :node_id => other_node.id, :reference_node_id => other_node.id)
      3.times { yesterday_other.differences.create! }

      2.times { @today.differences.create! }
      3.times { @yesterday.differences.create! }
      4.times { @old.differences.create! }

      BaselineUtil.other_day_summaries(@date, [@yesterday.id]).should == [
        ["#{@yesterday.date.to_s(:db)} (3 unreviewed)", @yesterday.date.to_s(:db)],
      ]
    end

    it "should not include dates without baselines" do
      4.times { @old.differences.create! }

      BaselineUtil.other_day_summaries(@date + 1.day).should == [
        [@today.date.to_s(:db), @today.date.to_s(:db)],
        [@yesterday.date.to_s(:db), @yesterday.date.to_s(:db)],
        ["#{@old.date.to_s(:db)} (4 unreviewed)", @old.date.to_s(:db)],
      ]
    end

    it "should return an empty array if there are no provided baselines" do
      BaselineUtil.other_day_summaries(@date, []).should == []
    end
  end
end
